package extensions.reports.itrium

import extensions.reports.generic.AbstractJasperFormField
import net.sf.jasperreports.engine.JRParameter
import play.twirl.api.Html
import utils.reports.{ FieldProperties => CommonFieldProperties }
import utils.reports.itrium.FieldProperties

class ElementFormField extends AbstractJasperFormField {

  private var elementType : Long = 0
  private var isMultiple : Boolean = false
  private var useTypeHierarchy : Boolean = false

  override def init(jrParameter: JRParameter): Unit = {
    super.init(jrParameter)
    elementType = getParameterProperty(jrParameter, FieldProperties.ELEMENT_TYPE).map(_.toLong).getOrElse(0L)
    useTypeHierarchy = getParameterProperty(jrParameter, FieldProperties.USE_TYPE_HIERACHY).map(_.toBoolean).getOrElse(false)
    isMultiple = getParameterProperty(jrParameter, CommonFieldProperties.MULTIPLE).map(_.toBoolean).getOrElse(false)
  }

  override def render(): Html = {
    views.html.reports.itrium.elementFormField(key(), elementType, useTypeHierarchy, isMultiple)
  }

  override def parse(value: String): AnyRef = {
    if (isMultiple) {
      value
    } else {
      try {
        Integer.valueOf(value)
      } catch {
        case t: Throwable =>
          null
      }
    }
  }
}