package extensions.reports.itrium

import extensions.reports.generic.AbstractJasperFormField
import net.sf.jasperreports.engine.JRParameter
import play.twirl.api.Html
import utils.reports.{ FieldProperties => CommonFieldProperties }
import utils.reports.itrium.FieldProperties

class EnumeratedPropertyFormField extends AbstractJasperFormField {

  private var propertyId : Long = 0
  private var isMultiple : Boolean = false

  override def init(jrParameter: JRParameter): Unit = {
    super.init(jrParameter)
    propertyId = getParameterProperty(jrParameter, FieldProperties.ENUMERATED_PROPERTY_ID).map(_.toLong).getOrElse(0L)
    isMultiple = getParameterProperty(jrParameter, CommonFieldProperties.MULTIPLE).map(_.toBoolean).getOrElse(false)
  }

  override def render(): Html = {
    views.html.reports.itrium.enumeratedPropertyFormField(key(), propertyId, isMultiple)
  }

  override def parse(value: String): AnyRef = {
    if (isMultiple) {
      value
    } else {
      try {
        Integer.valueOf(value)
      } catch {
        case t: Throwable =>
          null
      }
    }
  }
}
