package extensions.reports.itrium

import dto.reports.itrium.OrganizationDto
import extensions.reports.GenericFormField
import extensions.reports.generic.AbstractJasperFormField
import net.sf.jasperreports.engine.JRParameter
import play.twirl.api.Html
import utils.reports.itrium.FieldProperties

class OrganizationFormField extends AbstractJasperFormField {

  private var departmentsNotLinked : Boolean = false

  private var multipleOrganizations = false
  private var multipleDepartments = true

  def this(jrParameter: JRParameter, notLinked: Boolean) = {
    this()
    this.keyValue = jrParameter.getName
    this.titleValue = jrParameter.getDescription
    this.departmentsNotLinked = notLinked
  }

  def this(jRParameter: JRParameter) = {
    this(jRParameter, false)
  }

  override def init(jrParameter: JRParameter): Unit = {
    super.init(jrParameter)
    this.departmentsNotLinked = getParameterProperty(jrParameter, FieldProperties.ORGANIZATION_NOT_LINKED)
      .map(_.toBoolean).getOrElse(false)
    this.multipleOrganizations = getParameterProperty(jrParameter, FieldProperties.MULTIPLE_ORGANIZATIONS)
      .map(_.toBoolean).getOrElse(false)
    this.multipleDepartments = getParameterProperty(jrParameter, FieldProperties.MULTIPLE_DEPARTMENTS)
      .map(_.toBoolean).getOrElse(true)
  }

  override def render(): Html = {
    views.html.reports.itrium.organizationFormField(key(), departmentsNotLinked, multipleOrganizations, multipleDepartments)
  }

  override def parse(value: String): AnyRef = {
    val twoParts = value.split("/")
    if (twoParts.isEmpty) {
      new OrganizationDto("0", "")
    } else if (twoParts.length == 1) {
      new OrganizationDto(twoParts(0), "")
    } else {
      new OrganizationDto(twoParts(0), twoParts(1))
    }
  }
}
