package models.reports.itrium

import scala.collection.mutable.ListBuffer

case class Element(id: Long, name: String) {}

class ElementComponent() {
  def list(elementType: Long, useTypeHierarchy: Boolean)(implicit connection: java.sql.Connection): Iterable[Element] = {
    val sql = useTypeHierarchy match {
      case true =>
        s"""
          (
            SELECT e.element_id, e.element_name FROM elements AS e WHERE e.type_id = $elementType
          ) UNION (
            SELECT e.element_id, e.element_name FROM elements AS e
            JOIN types_link AS tl
              ON e.type_id = tl.child_id
            WHERE tl.parent_id = $elementType
          )
          ORDER BY element_name ASC
        """
      case false =>
        s"""
          SELECT e.element_id, e.element_name FROM elements AS e WHERE e.type_id = $elementType
          ORDER BY element_name ASC
        """
    }
    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(sql)

    val resultListBuffer = ListBuffer[Element]()

    while (resultSet.next()) {
      val id = resultSet.getInt("element_id")
      val name = resultSet.getString("element_name")
      resultListBuffer += Element(id, name)
    }

    resultListBuffer.toSeq
  }
}