package models.reports.itrium

import scala.collection.mutable.ListBuffer

case class EnumeratedPropertyOption(id: Long, value: String) {}

class EnumeratedPropertyOptionComponent() {
  def findByPropertyId(propertyId: Long)(implicit connection: java.sql.Connection): Iterable[EnumeratedPropertyOption] = {
    val sql = s"""
    WITH
    epe_id AS (
      SELECT e.element_id
        FROM elements AS e
        JOIN element_properties_int AS epi
          ON e.element_id = epi.element_id
        WHERE
          e.type_id = 4208 AND
          epi.property_id = 4204 AND
          epi.value = $propertyId
    ),
    epo_id AS (
      SELECT e.element_id
        FROM elements AS e
        JOIN elements_link el
          ON e.element_id = el.child_id AND el.parent_id IN (SELECT * FROM epe_id)
    )
    SELECT eps.element_id, eps.value FROM element_properties_string eps
      WHERE eps.property_id = ${propertyId + 1} AND eps.element_id IN (SELECT * FROM epo_id)
    UNION
    SELECT epi.element_id, epi.value::text FROM element_properties_int epi
      WHERE epi.property_id = ${propertyId + 1} AND epi.element_id IN (SELECT * FROM epo_id)
    """

    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(sql)

    val resultListBuffer = ListBuffer[EnumeratedPropertyOption]()

    while (resultSet.next()) {
      val id = resultSet.getInt("element_id")
      val value = resultSet.getString("value")
      resultListBuffer += EnumeratedPropertyOption(id, value)
    }

    resultListBuffer.toSeq
  }
}