package models.reports.itrium

import play.api.Logger

import scala.collection.mutable.ListBuffer

case class Organization(id: Long, name: String) {

}

case class Department(id: Long, name: String) {}

class OrganizationComponent {
  private val logger = Logger(getClass)

  def listOrganizations()(implicit connection: java.sql.Connection): Iterable[Organization] = {
    val sql = "SELECT element_id,element_name FROM elements WHERE type_id = 4241 ORDER BY element_name ASC"
    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(sql)

    val organizationListBuffer = ListBuffer[Organization]()

    while (resultSet.next()) {
      val id = resultSet.getInt("element_id")
      val name = resultSet.getString("element_name")
      organizationListBuffer += Organization(id, name)
    }

    organizationListBuffer.toSeq
  }

  def listDepartments()(implicit connection: java.sql.Connection): Iterable[Department] = {
    val sql = "SELECT element_id,element_name FROM elements WHERE type_id = 4242 ORDER BY element_name ASC"
    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(sql)

    val departmentListBuffer = ListBuffer[Department]()

    while (resultSet.next()) {
      val id = resultSet.getInt("element_id")
      val name = resultSet.getString("element_name")
      departmentListBuffer += Department(id, name)
    }

    departmentListBuffer.toSeq
  }

  def findDepartments(organizationId: Long)(implicit connection: java.sql.Connection): Iterable[Department] = {
    val result = ListBuffer[Department]()

    def findChildren(parentId: Long): Unit = {
      val sql = "SELECT element_id,element_name FROM elements AS e JOIN elements_link AS l " +
        s"ON e.element_id = l.child_id WHERE l.parent_id = $parentId AND e.type_id = 4242"
      val statement = connection.createStatement()
      val resultSet = statement.executeQuery(sql)

      val children = ListBuffer[Department]()

      while (resultSet.next()) {
        children += Department(resultSet.getLong("element_id"), resultSet.getString("element_name"))
      }

      children.foreach(d => {
        result += d
        findChildren(d.id)
      })
    }

    findChildren(organizationId)
    result.sortWith(_.name < _.name).toSeq
  }
}
