package models.reports.itrium

import java.math.BigInteger

import dto.reports.itrium.PassFilter
import play.api.Logger

import scala.collection.mutable
import scala.collection.mutable.ListBuffer

case class Person(id: Long, surname: String, name: String, patronymic: String) {}

case class Pass(id: Long, cardNumber: String, pincode: String, person: Person) {

}

class PassComponent {
  private val logger = Logger(getClass)

  def findPasses(filter: PassFilter)
                (implicit connection: java.sql.Connection): Iterable[Pass] = {
    val sqlQueryBuilder = mutable.ListBuffer[String]()
    sqlQueryBuilder += "SELECT element_id FROM elements WHERE type_id = 4206"

    if (filter.cardNumber != null && filter.cardNumber.nonEmpty) {
      try {
        val cardNumberBig = new BigInteger(filter.cardNumber)
        val cardNumberHex = cardNumberBig.toString(16).toUpperCase // this is the HEX

        sqlQueryBuilder +=
          s""" AND element_id IN (
              |  SELECT value FROM element_properties_ref AS person_ref WHERE person_ref.property_id = 4202 AND person_ref.element_id IN (
              |    SELECT element_id FROM element_properties_ref AS card_ref WHERE card_ref.property_id = 4201 AND card_ref.value IN (
              |      SELECT element_id FROM element_properties_string WHERE property_id = 6364 AND value = '$cardNumberHex'
              |    )
              |  )
              |)
         """.stripMargin
      } catch {
        case t: Throwable =>
          logger.warn(s"failed to parse card number '${filter.cardNumber}'?", t)
      }
    }

    if (filter.pincode != null && filter.pincode.nonEmpty) {
      sqlQueryBuilder +=
        s""" AND element_id IN (
            |  SELECT value FROM element_properties_ref AS person_ref WHERE person_ref.property_id = 4202 AND person_ref.element_id IN (
            |    SELECT element_id FROM element_properties_string AS pin WHERE pin.property_id = 4214 AND pin.value = '${filter.pincode}'
            |  )
            |)
         """.stripMargin
    }

    if (filter.surname != null && !filter.surname.isEmpty) {
      sqlQueryBuilder += stringPropertyFilter(4234, filter.surname)
    }
    if (filter.name != null && !filter.name.isEmpty) {
      sqlQueryBuilder += stringPropertyFilter(4235, filter.name)
    }
    if (filter.patronymic != null && !filter.patronymic.isEmpty) {
      sqlQueryBuilder += stringPropertyFilter(4236, filter.patronymic)
    }

    sqlQueryBuilder += " LIMIT 100"

    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(sqlQueryBuilder.mkString(""))
    var elementIds = mutable.ListBuffer[Long]()
    while (resultSet.next()) {
      elementIds += resultSet.getLong("element_id")
    }

    logger.debug(s"found elements $elementIds; fetching information")


    val passes = ListBuffer[Pass]()
    elementIds.foreach(elementId => {
      val person = fetchPerson(elementId)
      passes ++= findPassesForPerson(person)
    })

    passes.toSeq
  }

  private def stringPropertyFilter(propertyId: Long, pattern: String): String = {
    s" AND element_id IN (SELECT element_id FROM element_properties_string WHERE property_id = $propertyId and value ILIKE '$pattern%')"
  }

  private def fetchPerson(personId: Long)
                         (implicit connection: java.sql.Connection): Person = {
    val query = s"SELECT property_id, value FROM element_properties_string WHERE element_id = $personId AND property_id IN (4234, 4235, 4236)"

    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(query)

    var surname: String = null
    var name: String = null
    var patronymic: String = null

    while (resultSet.next()) {
      val propertyValue = resultSet.getString("value")
      resultSet.getLong("property_id") match {
        case 4234 => surname = propertyValue
        case 4235 => name = propertyValue
        case 4236 => patronymic = propertyValue
      }
    }

    Person(personId, surname, name, patronymic)
  }

  private def fetchPass(passId: Long, person: Person)
                       (implicit connection: java.sql.Connection): Pass = {
    val pincode: String = {
      val query = s"SELECT value FROM element_properties_string WHERE element_id = $passId AND property_id = 4214"
      val statement = connection.createStatement()
      val resultSet = statement.executeQuery(query)
      if (resultSet.next()) { resultSet.getString("value") } else { "" }
    }

    val cardIdOption: Option[Long] = {
      val query = s"SELECT value FROM element_properties_ref WHERE element_id = $passId AND property_id = 4201"
      val statement = connection.createStatement()
      val resultSet = statement.executeQuery(query)
      if (resultSet.next()) { Some(resultSet.getLong("value")) } else { None }
    }

    val cardNumber = cardIdOption match {
      case Some(cardId) =>
        val query = s"SELECT value FROM element_properties_string WHERE element_id = $cardId AND property_id = 6364"
        val statement = connection.createStatement()
        val resultSet = statement.executeQuery(query)
        if (resultSet.next()) {
          new BigInteger(resultSet.getString("value"), 16).toString()
        } else { "" }
      case None => ""
    }

    Pass(passId, cardNumber, pincode, person)
  }

  // 4202
  private def findPassesForPerson(person: Person)
                                 (implicit connection: java.sql.Connection): Iterable[Pass] = {
    val query = s"SELECT element_id FROM element_properties_ref WHERE value = ${person.id} AND property_id = 4202"
    val statement = connection.createStatement()
    val resultSet = statement.executeQuery(query)

    val passIds = mutable.ListBuffer[Long]()
    while (resultSet.next()) {
      passIds += resultSet.getLong("element_id")
    }

    passIds.map(id => {
      fetchPass(id, person)
    })
  }
}
